//
//  MedicalCertificateConfiguration.swift
//  scanbot_sdk
//
//  Created by Seifeddine Bouzid on 28.03.23.
//

import Foundation
import ScanbotSDK

struct MedicalCertificateConfiguration {
    
    var flashEnabled: Bool?
    var zoomConfiguration: SBSDKBaseScannerZoomConfiguration?
    var viewFinderConfiguration: SBSDKBaseScannerViewFinderConfiguration?
    var scannerConfiguration: ScannerConfiguration?
    
    init() { }
    
    init(_ arguments: [String: Any?]?) {
        if let arguments {
            self = MedicalCertificateConfiguration.parseArguments(arguments)
        }
    }
    
    private static func parseArguments(_ arguments: [String: Any?]) -> MedicalCertificateConfiguration {
        var configuration = MedicalCertificateConfiguration()
        
        if let flashEnabled = arguments["flashEnabled"] as? Bool {
            configuration.flashEnabled = flashEnabled
        }
        
        if let cameraZoomFactor = arguments["cameraZoomFactor"] as? CGFloat {
            let zoomConfig = SBSDKBaseScannerZoomConfiguration.default()
            zoomConfig.initialZoomFactor = cameraZoomFactor
            configuration.zoomConfiguration = zoomConfig
        }
        
        if let finderConfiguration: [String: Any] = arguments["finder"] as? [String: Any] {
            configuration.viewFinderConfiguration = self.configureFinderView(finderConfiguration)
        }
        
        if let scannerConfiguration = arguments["scannerConfiguration"] as? [String: Any] {
            configuration.scannerConfiguration = ScannerConfiguration(scannerConfiguration)
        }
        return configuration
    }
    
    private static func configureFinderView(_ configuration: [String: Any?]) -> SBSDKBaseScannerViewFinderConfiguration {
        let finderConfiguration = SBSDKBaseScannerViewFinderConfiguration.default()
        
        finderConfiguration.backgroundColor = UIColor(hexString: "#00000000")
        finderConfiguration.lineColor = UIColor(hexString: "#00000000")
        finderConfiguration.isViewFinderEnabled = true
        
        if let finderAspectRatio = configuration["finderAspectRatio"] as? [String: Any],
           let width = finderAspectRatio["width"] as? Double,
           let height = finderAspectRatio["height"] as? Double
        {
            finderConfiguration.aspectRatio = SBSDKAspectRatio(width: width, andHeight: height)
        }
        
        if let insets = configuration["finderInsets"] as? [String: Any] {
            if let left = insets["left"] as? CGFloat {
                finderConfiguration.minimumInset.left = left
            }
            
            if let top = insets["top"] as? CGFloat {
                finderConfiguration.minimumInset.top = top
            }
            
            if let right = insets["right"] as? CGFloat {
                finderConfiguration.minimumInset.right = right
            }
            
            if let bottom = insets["bottom"] as? CGFloat {
                finderConfiguration.minimumInset.bottom = bottom
            }
        }
        return finderConfiguration
    }
    
    struct ScannerConfiguration {
        
        var captureHighResolutionImage: Bool?
        var extractPatientInfo: Bool?
        var extractBarcode: Bool?
        var recognizeBarcode: Bool?
        
        init() { }
        
        init(_ arguments: [String: Any]) {
            self = ScannerConfiguration.parseArguments(arguments: arguments)
        }
        
        private static func parseArguments(arguments: [String: Any]) -> ScannerConfiguration {
            var configuration = ScannerConfiguration()
            
            if let captureHighResolutionImage = arguments["captureHighResolutionImage"] as? Bool {
                configuration.captureHighResolutionImage = captureHighResolutionImage
            }
            
            if let extractPatientInfo = arguments["recognizePatientInfo"] as? Bool {
                configuration.extractPatientInfo = extractPatientInfo
            }
            
            if let extractBarcode = arguments["recognizeBarcode"] as? Bool {
                configuration.extractBarcode = extractBarcode
            }
            return configuration
        }
    }
}
