//
//  SBSDKUIBarcodesBatchScannerBehaviorConfiguration.swift
//  barcode_scanner
//
//  Created by Yevgeniy Knizhnik on 20.01.21.
//

import Foundation
import ScanbotSDK

extension SBSDKUIBarcodesBatchScannerBehaviorConfiguration {
    convenience init(from dictionary: [String: Any]) {
        self.init()
        
        if let flashEnabled = dictionary["flashEnabled"] as? Bool {
            self.isFlashEnabled = flashEnabled
        }
        
        if let successBeepEnabled = dictionary["successBeepEnabled"] as? Bool {
            self.isSuccessBeepEnabled = successBeepEnabled
        }
        
        if let value = dictionary["additionalParameters"] as? [String: Any] {
            let barcodeImageGenerationType = SBSDKBarcodeAdditionalParameters(from: value)
            self.additionalDetectionParameters = barcodeImageGenerationType
        }
        
        if let value = dictionary["gs1DecodingEnabled"] as? Bool {
            self.additionalDetectionParameters.enableGS1Decoding = value
        }
        
        if let value = dictionary["minimumTextLength"] as? UInt {
            self.additionalDetectionParameters.minimumTextLength = value
        }
        
        if let value = dictionary["maximumTextLength"] as? UInt {
            self.additionalDetectionParameters.maximumTextLength = value
        }
        
        if let value = dictionary["minimum1DBarcodesQuietZone"] as? UInt {
            self.additionalDetectionParameters.minimum1DBarcodesQuietZone = value
        }
        
        if let value = dictionary["msiPlesseyChecksumAlgorithm"] as? String {
            self.additionalDetectionParameters.msiPlesseyChecksumAlgorithm = SBSDKBarcodeMSIPlesseyChecksumAlgorithm(from: value)
        }
        
        if let value = dictionary["stripCheckDigits"] as? Bool {
            self.additionalDetectionParameters.stripCheckDigits = value
        }

        if let focusLockEnabled = dictionary["minFocusDistanceLock"] as? Bool {
            self.isFocusLockEnabled = focusLockEnabled
        }
        
        if let engineMode = SBSDKBarcodeEngineMode.fromArguments(dictionary) {
            self.engineMode = engineMode
        }
        
        if dictionary["barcodeFormats"] is [String] {
            self.acceptedBarcodeTypes = SBSDKBarcodeType.fromArguments(dictionary) ?? []
        }
        
        if dictionary["acceptedDocumentFormats"] is [String] {
            self.acceptedDocumentTypes = SBSDKBarcodeDocumentType.fromArguments(dictionary) ?? []
        }
    }
}
