import 'package:flutter/cupertino.dart';
import 'document_live_detection.dart';
import 'document_scanner_configuration.dart';
import 'classical_camera.dart';

export 'document_live_detection.dart';
export 'document_scanner_configuration.dart';
export 'classical_camera.dart';

class DocumentScannerCamera extends StatefulWidget {
  const DocumentScannerCamera(
      {Key? key,
      required this.cameraDetector,
      required this.configuration,
      required this.onWidgetReady,
      this.onCameraPreviewStarted,
      this.onHeavyOperationProcessing})
      : super(key: key);

  /// The detector that will be used to detect documents and return the results or errors from it.
  final DocumentCameraLiveDetector cameraDetector;

  /// The configuration of the camera and detection behavior.
  final DocumentCameraConfiguration configuration;

  /// Runs when the camera is ready to be used. This happens not only once when the widget is created but also when the app returns to foreground and user pops to camera page from another page, if `ScanbotCamera.scanbotSdkRouteObserver` is registered
  final Function(ScanbotCameraController controller) onWidgetReady;

  /// Runs each time the camera preview is started. This differs for android and ios.
  /// In android  this function will be called after each successful snap while in ios only once.
  /// As ios not stopping camera preview after the snap
  final Function()? onCameraPreviewStarted;

  /// triggers when camera plugin make long work eg. saving snap to the image etc.
  final Function(bool)? onHeavyOperationProcessing;

  @override
  State<DocumentScannerCamera> createState() => _DocumentScannerCameraState();
}

class _DocumentScannerCameraState extends State<DocumentScannerCamera> {
  @override
  void didUpdateWidget(covariant DocumentScannerCamera oldWidget) {
    var autoSnapEnabledNew =
        this.widget.configuration.scannerConfiguration.autoSnapEnabled;
    if (autoSnapEnabledNew !=
        oldWidget.configuration.scannerConfiguration.autoSnapEnabled) {
      this.widget.cameraDetector.setAutoSnappingEnabled(autoSnapEnabledNew);
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return ScanbotCamera(
      detector: widget.cameraDetector,
      // Here we set initial configurations of the camera
      configuration: widget.configuration,
      onCameraReady: widget.onWidgetReady,
      onPictureTaken: widget.cameraDetector.snapListener,
      onCameraPreviewStarted: widget.onCameraPreviewStarted,
      finder: null,
      onHeavyOperationProcessing: widget.onHeavyOperationProcessing,
    );
  }
}
